@@ -14,7 +14,7 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* Copyright (C) 2004-2022 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
+/* Copyright (C) 2004-2023 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
 /* Copyright (C) 2013 Cristiano Fontana <fontanacl@ornl.gov> */
 
 #define PROG "ssocr"
@@ -23,7 +23,7 @@
 #define SSOCR2_DEFINES_H
 
 /* version number */
-#define VERSION "2.22.1"
+#define VERSION "2.23.1"
 
 /* states */
 #define FIND_DARK 0
@@ -113,9 +113,21 @@
 /* add space characters if digit distance is greater than SPC_FAC * min dist */
 #define SPC_FAC 1.4
 
-/* to find segment need # of pixels */
+/* number of set pixels needed in a scanline to recognize a segment */
 #define NEED_PIXELS 1
 
+/* minimum number of pixels required for a segment
+ * this generalizes to both scanline and ratio based digit detection,
+ * but only when the digit is built using segments, i.e., this is not
+ * used for decimal separators */
+#define MIN_SEGMENT 1
+
+/* minimum width of a character respectively digit */
+#define MIN_CHAR_W 1
+
+/* minimum height of a character respectively digit */
+#define MIN_CHAR_H 1
+
 /* ignore # of pixels when checking a column fo black or white */
 #define IGNORE_PIXELS 0
 
@@ -172,7 +184,7 @@ typedef enum luminance_e {
   BLUE
 } luminance_t;
 
-/* direction, to mirror horizontally or vertically, or for a scan line */
+/* direction, to mirror horizontally or vertically, or for a scanline */
 typedef enum direction_e {
   HORIZONTAL,
   VERTICAL
