@@ -14,7 +14,7 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* Copyright (C) 2004-2022 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
+/* Copyright (C) 2004-2023 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
 /* Copyright (C) 2013 Cristiano Fontana <fontanacl@ornl.gov> */
 
 /* standard things */
@@ -92,7 +92,7 @@ void print_version(FILE *f)
 {
   fprintf(f, "Seven Segment Optical Character Recognition Version %s\n",
              VERSION);
-  fprintf(f, "Copyright (C) 2004-2022 by Erik Auerswald"
+  fprintf(f, "Copyright (C) 2004-2023 by Erik Auerswald"
              " <auerswal@unix-ag.uni-kl.de>\n");
   fprintf(f, "This program comes with ABSOLUTELY NO WARRANTY\n");
   fprintf(f, "This is free software, and you are welcome to redistribute it"
@@ -120,9 +120,12 @@ void usage(char *name, FILE *f)
   fprintf(f, "         -a, --absolute-threshold don't adjust threshold to image\n");
   fprintf(f, "         -T, --iter-threshold     use iterative thresholding method\n");
   fprintf(f, "         -n, --number-pixels=#    number of pixels needed to recognize a segment\n");
+  fprintf(f, "         -N, --min-segment=SIZE   minimum width and height of a segment\n");
   fprintf(f, "         -i, --ignore-pixels=#    number of pixels ignored when searching digit\n");
   fprintf(f, "                                  boundaries\n");
-  fprintf(f, "         -d, --number-digits=#    number of digits in image (-1 for auto)\n");
+  fprintf(f, "         -M, --min-char-dims=WxH  minimum width and height of a character/digit\n");
+  fprintf(f, "         -d, --number-digits=RNG  number of digits in image (-1 for auto,\n");
+  fprintf(f, "                                  positive number, or positive range)\n");
   fprintf(f, "         -r, --one-ratio=#        height/width ratio to recognize a \'one\'\n");
   fprintf(f, "         -m, --minus-ratio=#      width/height ratio to recognize a minus sign\n");
   fprintf(f, "         -H, --dec-h-ratio=#      max_dig_h/h ratio to recognize decimal point\n");
@@ -184,22 +187,26 @@ void usage(char *name, FILE *f)
   fprintf(f, "                                  pixels set (including checked position)\n");
   fprintf(f, "          keep_pixels_filter MASK keeps pixels that have at least MASK neighbor\n");
   fprintf(f, "                                  pixels set (not counting the checked pixel)\n");
-  fprintf(f, "\nDefaults: needed pixels          = %2d\n", NEED_PIXELS);
-  fprintf(f, "          ignored pixels         = %2d\n", IGNORE_PIXELS);
-  fprintf(f, "          no. of digits          = %2d\n", NUMBER_OF_DIGITS);
-  fprintf(f, "          threshold              = %5.2f\n", THRESHOLD);
-  fprintf(f, "          foreground             = %s\n",
+  fprintf(f, "\nDefaults: needed pixels            = %2d\n", NEED_PIXELS);
+  fprintf(f, "          minimum segment size     = %2d\n", MIN_SEGMENT);
+  fprintf(f, "          minimum character width  = %2d\n", MIN_CHAR_W);
+  fprintf(f, "          minimum character height = %2d\n", MIN_CHAR_H);
+  fprintf(f, "          ignored pixels           = %2d\n", IGNORE_PIXELS);
+  fprintf(f, "          minimum number of digits = %2d\n", NUMBER_OF_DIGITS);
+  fprintf(f, "          maximum number of digits = %2d\n", NUMBER_OF_DIGITS);
+  fprintf(f, "          threshold                = %5.2f\n", THRESHOLD);
+  fprintf(f, "          foreground               = %s\n",
       (SSOCR_DEFAULT_FOREGROUND == SSOCR_BLACK) ? "black" : "white");
-  fprintf(f, "          background             = %s\n",
+  fprintf(f, "          background               = %s\n",
       (SSOCR_DEFAULT_BACKGROUND == SSOCR_BLACK) ? "black" : "white");
-  fprintf(f, "          luminance              = ");
+  fprintf(f, "          luminance                = ");
   print_lum_key(DEFAULT_LUM_FORMULA, f); fprintf(f, "\n");
-  fprintf(f, "          height/width threshold = %2d\n", ONE_RATIO);
+  fprintf(f, "          height/width threshold for digit one  = %2d\n", ONE_RATIO);
   fprintf(f, "          width/height threshold for minus sign = %2d\n", MINUS_RATIO);
   fprintf(f, "          max_dig_h/h threshold for decimal sep = %2d\n", DEC_H_RATIO);
   fprintf(f, "          max_dig_w/w threshold for decimal sep = %2d\n", DEC_W_RATIO);
-  fprintf(f, "          space width factor     = %.2f\n", SPC_FAC);
-  fprintf(f, "          character set          = ");
+  fprintf(f, "          space width factor       = %.2f\n", SPC_FAC);
+  fprintf(f, "          character set            = ");
   print_cs_key(DEFAULT_CHARSET, f); fputs("\n", f);
   fprintf(f, "\nOperation: The IMAGE is read, the COMMANDs are processed in the sequence\n");
   fprintf(f, "           they are given, in the resulting image the given number of digits\n");
