@@ -14,7 +14,7 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* Copyright (C) 2004-2022 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
+/* Copyright (C) 2004-2023 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
 /* Copyright (C) 2013 Cristiano Fontana <fontanacl@ornl.gov> */
 
 /* ImLib2 Header */
@@ -22,12 +22,13 @@
 #include <Imlib2.h>
 
 /* standard things */
+#include <limits.h>         /* INT_MAX */
 #include <stdint.h>         /* SIZE_MAX */
 #include <stdio.h>          /* puts, printf, BUFSIZ, perror, FILE */
 #include <stdlib.h>         /* exit */
 
 /* string manipulation */
-#include <string.h>         /* memcpy, strdup, strlen */
+#include <string.h>         /* memcpy, strchr, strdup, strlen */
 
 /* option parsing */
 #include <getopt.h>         /* getopt */
@@ -75,6 +76,10 @@ static char * tmp_imgfile(unsigned int flags)
   }
   dir_len = strlen(dir);
   pattern_len = dir_len + pat_suffix_len + 1;
+  if(pattern_len <= dir_len) {
+    fputs(PROG ": error: temporary file name length overflow\n", stderr);
+    exit(99);
+  }
   name = calloc(pattern_len, sizeof(char));
   if(!name) {
     perror(PROG ": could not allocate memory for name of temporary file");
@@ -108,7 +113,7 @@ static char * tmp_imgfile(unsigned int flags)
   return name;
 }
 
-/* return number of foreground pixels in a scan line */
+/* return number of foreground pixels in a scanline */
 static unsigned int scanline(Imlib_Image *image, Imlib_Image *debug_image,
                              int x, int y, int len, direction_t dir,
                              color_struct d_color, double thresh,
@@ -146,6 +151,91 @@ static void print_spaces(FILE *f, int n)
   }
 }
 
+/* parse dimensions given as a string in the format "WxH" */
+static int parse_width_height(const char *s, dimensions_struct *d)
+{
+  size_t l;
+  const char *width_string;
+  char *height_string;
+  int w, h;
+  if (!s || !d) {
+    fputs(PROG ": error: parse_width_height() called with NULL pointer\n",
+          stderr);
+    return 1;
+  }
+  l = strlen(s);
+  if (l == 0) {
+    fputs(PROG ": error: parse_width_height() called with empty string\n",
+          stderr);
+    return 1;
+  }
+  width_string = s;
+  height_string = strchr(s, 'x');
+  if (!height_string) {
+    fputs(PROG ": error: no 'x' in dimension specification\n", stderr);
+    return 1;
+  }
+  if (width_string == height_string) {
+    fputs(PROG ": error: width missing from dimension specification\n", stderr);
+    return 1;
+  }
+  height_string++;
+  if (strlen(height_string) == 0) {
+    fputs(PROG ": error: height missing from dimension specification\n",stderr);
+    return 1;
+  }
+  w = atoi(width_string);
+  h = atoi(height_string);
+  if (w < 1 || h < 1) {
+    fprintf(stderr,
+            PROG ": warning: ignoring mininmum character dimensions %dx%d\n",
+            w, h);
+    return 1;
+  }
+  d->w = w;
+  d->h = h;
+  return 0;
+}
+
+/* parse description of interval in one of two formats:
+ * 1) a single number giving both upper and lower bound
+ * 2) two numbers separated by a hyphen */
+static int parse_interval(const char *s, interval_struct *i) {
+  int min, max;
+  char *upper;
+
+  if (!s) {
+    fputs(PROG ": error: empty interval description string\n", stderr);
+    return 1;
+  }
+  min = atoi(s);
+  if (min == -1) {
+    i->min = i->max = min;
+    return 0;
+  }
+  if (min < 1) {
+    fputs(PROG ": error: lower interval bound cannot be less than 1\n", stderr);
+    return 1;
+  }
+  upper = strchr(s, '-');
+  if (!upper) {
+    i->min = i->max = min;
+    return 0;
+  }
+  if (upper == s) {
+    fputs(PROG ": error: lower interval bound cannot be omitted\n", stderr);
+    return 1;
+  }
+  max = atoi(++upper);
+  if (max < min) {
+    fputs(PROG ": error: upper bound less than lower bound\n", stderr);
+    return 1;
+  }
+  i->min = min;
+  i->max = max;
+  return 0;
+}
+
 /*** main() ***/
 
 int main(int argc, char **argv)
@@ -158,9 +248,14 @@ int main(int argc, char **argv)
   int use_tmpfile=0; /* flag to know if temporary image file is used */
 
   int i, j, d;  /* iteration variables */
+  size_t cur_digit_mem, new_digit_mem; /* for overflow checks */
   int unknown_digit=0; /* was one of the 6 found digits an unknown one? */
-  int need_pixels = NEED_PIXELS; /*pixels needed to recognize a segment as set*/
-  int number_of_digits = NUMBER_OF_DIGITS; /* look for this many digits */
+  int need_pixels = NEED_PIXELS; /* pixels needed to set segment in scanline */
+  int min_segment = MIN_SEGMENT; /* minimum pixels needed for a segment */
+  dimensions_struct min_char_dims; /* minimum character dimensions (W x H) */
+  int potential_digits; /* number of potential digits after segmentation */
+  interval_struct expected_digits; /* expect number of digits is inside this */
+  int number_of_digits; /* number of digits found and accepted */
   int ignore_pixels = IGNORE_PIXELS; /* pixels to ignore when checking column */
   int one_ratio = ONE_RATIO; /* height/width > one_ratio => digit 'one' */
   int minus_ratio = MINUS_RATIO; /* height/width > minus_ratio => char 'minus'*/
@@ -190,6 +285,11 @@ int main(int argc, char **argv)
   int found_pixels=0; /* how many pixels are already found */
   color_struct d_color = {0, 0, 0, 0}; /* drawing color */
 
+  /* initialize structures */
+  min_char_dims.w = MIN_CHAR_W;
+  min_char_dims.h = MIN_CHAR_H;
+  expected_digits.min = expected_digits.max = NUMBER_OF_DIGITS;
+
   /* if we provided no arguments to the program exit */
   if (argc < 2) {
     usage(PROG, stderr);
@@ -208,6 +308,8 @@ int main(int argc, char **argv)
       {"absolute-threshold", 0, 0, 'a'}, /* use treshold value as provided */
       {"iter-threshold", 0, 0, 'T'}, /* use treshold value as provided */
       {"number-pixels", 1, 0, 'n'}, /* pixels needed to regard segment as set */
+      {"min-segment", 1, 0, 'N'}, /* minimum pixels needed for a segment */
+      {"min-char-dims", 1, 0, 'M'}, /* minimum character (digit) dimensions */
       {"ignore-pixels", 1, 0, 'i'}, /* pixels ignored when searching digits */
       {"number-digits", 1, 0, 'd'}, /* number of digits in image */
       {"one-ratio", 1, 0, 'r'}, /* height/width threshold to recognize a one */
@@ -234,7 +336,7 @@ int main(int argc, char **argv)
       {0, 0, 0, 0} /* terminate long options */
     };
     c = getopt_long (argc, argv,
-                     "hVt:vaTn:i:d:r:m:o:O:D::pPf:b:Igl:SXCc:H:W:sA:G",
+                     "hVt:vaTn:N:i:d:r:m:M:o:O:D::pPf:b:Igl:SXCc:H:W:sA:G",
                      long_options, &option_index);
     if (c == -1) break; /* leaves while (1) loop */
     switch (c) {
@@ -280,6 +382,39 @@ int main(int argc, char **argv)
             fprintf(stderr, "warning: ignoring --number-pixels=%s\n", optarg);
             need_pixels = NEED_PIXELS;
           }
+          if(flags & DEBUG_OUTPUT) {
+            fprintf(stderr, "need_pixels = %d\n", need_pixels);
+          }
+        }
+        break;
+      case 'N':
+        if(optarg) {
+          min_segment = atoi(optarg);
+          if(min_segment < 1) {
+            fprintf(stderr, "warning: ignoring --min-segment=%s\n", optarg);
+            min_segment = MIN_SEGMENT;
+            if(flags & DEBUG_OUTPUT) {
+              fprintf(stderr, "min_segment = %d\n", min_segment);
+            }
+          } else {
+            need_pixels = min_segment;
+            if(flags & DEBUG_OUTPUT) {
+              fprintf(stderr, "min_segment = need_pixels = %d\n", min_segment);
+            }
+          }
+        }
+        break;
+      case 'M':
+        if(optarg) {
+          int ret;
+          ret = parse_width_height(optarg, &min_char_dims);
+          if (ret) {
+            fprintf(stderr, "warning: ignoring --min-char-dims=%s\n", optarg);
+          }
+          if(flags & DEBUG_OUTPUT) {
+            fprintf(stderr, "min_char_dims = %dx%d\n", min_char_dims.w,
+                    min_char_dims.h);
+          }
         }
         break;
       case 'i':
@@ -293,10 +428,15 @@ int main(int argc, char **argv)
         break;
       case 'd':
         if(optarg) {
-          number_of_digits = atoi(optarg);
-          if((number_of_digits < 1) && (number_of_digits != -1)) {
+          int ret;
+          ret = parse_interval(optarg, &expected_digits);
+          if(ret) {
             fprintf(stderr, "warning: ignoring --number-digits=%s\n", optarg);
-            number_of_digits = NUMBER_OF_DIGITS;
+            expected_digits.min = expected_digits.max = NUMBER_OF_DIGITS;
+          }
+          if (flags & DEBUG_OUTPUT) {
+            fprintf(stderr, "expected_digits.min = %d\n", expected_digits.min);
+            fprintf(stderr, "expected_digits.max = %d\n", expected_digits.max);
           }
         }
         break;
@@ -492,8 +632,11 @@ int main(int argc, char **argv)
     fprintf(stderr, "flags & PRINT_SPACES=%d\n", flags & PRINT_SPACES);
     fprintf(stderr, "flags & SPC_USE_AVG_DST=%d\n", flags & SPC_USE_AVG_DST);
     fprintf(stderr, "need_pixels = %d\n", need_pixels);
+    fprintf(stderr, "min_segment = %d\n", min_segment);
+    fprintf(stderr, "min_char_dims = %dx%d\n",min_char_dims.w,min_char_dims.h);
     fprintf(stderr, "ignore_pixels = %d\n", ignore_pixels);
-    fprintf(stderr, "number_of_digits = %d\n", number_of_digits);
+    fprintf(stderr, "expected_digits.min = %d\n", expected_digits.min);
+    fprintf(stderr, "expected_digits.max = %d\n", expected_digits.max);
     fprintf(stderr, "foreground = %d (%s)\n", ssocr_foreground,
                     (ssocr_foreground == SSOCR_BLACK) ? "black" : "white");
     fprintf(stderr, "background = %d (%s)\n", ssocr_background,
@@ -502,8 +645,8 @@ int main(int argc, char **argv)
     print_lum_key(lt, stderr); fprintf(stderr, "\n");
     fprintf(stderr, "charset    = ");
     print_cs_key(charset, stderr); fprintf(stderr, "\n");
-    fprintf(stderr, "height/width threshold for one   = %d\n", one_ratio);
-    fprintf(stderr, "width/height threshold for minus = %d\n", minus_ratio);
+    fprintf(stderr, "height/width threshold for one    = %d\n", one_ratio);
+    fprintf(stderr, "width/height threshold for minus  = %d\n", minus_ratio);
     fprintf(stderr, "max_dig_h/h threshold for decimal = %d\n", dec_h_ratio);
     fprintf(stderr, "max_dig_w/w threshold for decimal = %d\n", dec_w_ratio);
     fprintf(stderr, "distance factor for adding spaces = %.2f\n", spc_fac);
@@ -952,17 +1095,16 @@ int main(int argc, char **argv)
     debug_image = make_mono(&image, thresh, lt);
   }
 
-  /* allocate memory for seven segment digits */
-  if(number_of_digits > -1) {
-    if(!(digits = calloc(number_of_digits, sizeof(digit_struct)))) {
-      perror(PROG ": digits = calloc()");
-      exit(99);
-    }
-  } else {
-    if(!(digits = calloc(1, sizeof(digit_struct)))) {
-      perror(PROG ": digits = calloc()");
-      exit(99);
-    }
+  /* start image segmentation into possible characters / digits */
+  if (flags & DEBUG_OUTPUT) {
+    fputs("starting image segmentation\n", stderr);
+    fputs("starting horizontal partitioning\n", stderr);
+  }
+
+  /* allocate memory for one seven segment digit */
+  if(!(digits = calloc(1, sizeof(digit_struct)))) {
+    perror(PROG ": digits = calloc()");
+    exit(99);
   }
 
   /* horizontal partition */
@@ -978,7 +1120,7 @@ int main(int argc, char **argv)
       if(is_pixel_set(lum, thresh)) /* dark */ {
         found_pixels++;
         if(found_pixels > ignore_pixels) {
-          /* 1 dark pixels darken the whole column */
+          /* 1 not ignored dark pixel darkens the whole column */
           col = (ssocr_foreground == SSOCR_BLACK) ? DARK : LIGHT;
         }
       } else if(col == UNKNOWN) /* light */ {
@@ -989,15 +1131,9 @@ int main(int argc, char **argv)
     if((state == ((ssocr_foreground == SSOCR_BLACK) ? FIND_DARK : FIND_LIGHT))
         && (col == ((ssocr_foreground == SSOCR_BLACK) ? DARK : LIGHT))) {
       /* beginning of digit */
-      if((number_of_digits > -1) && (d >= number_of_digits)) {
-        fprintf(stderr, "found too many digits (%d)\n", d+1);
-        imlib_free_image_and_decache();
-        if(flags & USE_DEBUG_IMAGE) {
-          save_image("debug", debug_image, output_fmt,debug_image_file,flags);
-          imlib_context_set_image(debug_image);
-          imlib_free_image_and_decache();
-        }
-        exit(1);
+      if (flags & DEBUG_OUTPUT) {
+        fprintf(stderr, " start of potential digit %d in image column %d\n",
+                d, i);
       }
       digits[d].x1 = i;
       digits[d].y1 = 0;
@@ -1012,8 +1148,16 @@ int main(int argc, char **argv)
               ((ssocr_foreground == SSOCR_BLACK) ? FIND_LIGHT : FIND_DARK))
               && (col == ((ssocr_foreground == SSOCR_BLACK) ? LIGHT : DARK))){
       /* end of digit */
+      if (flags & DEBUG_OUTPUT) {
+        fprintf(stderr, " end of potential digit %d in image column %d\n",d,i);
+      }
       digits[d].x2 = i;
       digits[d].y2 = h-1;
+      if((d >= INT_MAX - 1) || (d < 0)) {
+        fputs(PROG ": error: too many potential digits (integer overflow)\n",
+              stderr);
+        exit(99);
+      }
       d++;
       if(flags & USE_DEBUG_IMAGE) {
         imlib_context_set_image(debug_image);
@@ -1021,8 +1165,14 @@ int main(int argc, char **argv)
         imlib_image_draw_line(i,0,i,h-1,0);
         imlib_context_set_image(image);
       }
-      /* if number of digits is not known, add memory for another digit */
-      if(!(digits = realloc(digits, (d+1) * sizeof(digit_struct)))) {
+      /* add memory for another digit */
+      cur_digit_mem = d * sizeof(digit_struct);
+      new_digit_mem = (d+1) * sizeof(digit_struct);
+      if(new_digit_mem <= cur_digit_mem) {
+        fputs(PROG ": error: size_t overflow (memory for digits)\n", stderr);
+        exit(99);
+      }
+      if(!(digits = realloc(digits, new_digit_mem))) {
         perror(PROG ": digits = realloc()");
         exit(99);
       }
@@ -1037,31 +1187,27 @@ int main(int argc, char **argv)
    * if it is still searching for light then end the digit at the border of the
    * image */
   if(state == (ssocr_foreground == SSOCR_BLACK) ? FIND_LIGHT : FIND_DARK) {
+    if (flags & DEBUG_OUTPUT) {
+      fprintf(stderr, " end of potential digit %d in image column %d\n",d,w-1);
+    }
     digits[d].x2 = w-1;
     digits[d].y2 = h-1;
     d++;
     state = (ssocr_foreground == SSOCR_BLACK) ? FIND_DARK : FIND_LIGHT;
   }
-  if((number_of_digits > -1) && (d != number_of_digits)) {
-    fprintf(stderr, "found only %d of %d digits\n", d, number_of_digits);
-    imlib_free_image_and_decache();
-    if(flags & USE_DEBUG_IMAGE) {
-      save_image("debug", debug_image, output_fmt, debug_image_file, flags);
-      imlib_context_set_image(debug_image);
-      imlib_free_image_and_decache();
-    }
-    exit(1);
-  } else if(number_of_digits == -1) {
-    number_of_digits = d;
-    if(flags & DEBUG_OUTPUT) {
-      fprintf(stderr, "auto detecting number of digits: %d\n", d);
-    }
+
+  /* horizontal partitioning has found "d" potential characters / digits */
+  potential_digits = d;
+  if(flags & DEBUG_OUTPUT) {
+    fprintf(stderr, "horizontal partitioning found %d digit(s)\n",
+            potential_digits);
   }
-  dig_w = digits[number_of_digits-1].x2 - digits[0].x1;
-  dig_h = digits[number_of_digits-1].y2 - digits[0].y1;
 
   /* find upper and lower boundaries of every digit */
-  for(d=0; d<number_of_digits; d++) {
+  if (flags & DEBUG_OUTPUT) {
+    fputs("looking for upper and lower digit boundaries\n", stderr);
+  }
+  for(d=0; d<potential_digits; d++) {
     int found_top=0;
     state = (ssocr_foreground == SSOCR_BLACK) ? FIND_DARK : FIND_LIGHT;
     /* start from top of image and scan rows for dark pixel(s) */
@@ -1136,8 +1282,95 @@ int main(int argc, char **argv)
       }
     }
   }
+  if (flags & DEBUG_OUTPUT) {
+    fprintf(stderr, "image segmentation found %d potential digits\n",
+            potential_digits);
+  }
+
+  /* image has been segmented into potential digits, ignore too small ones */
+  if (min_char_dims.w > 1 || min_char_dims.h > 1) {
+    int digit_count = 0, pos;
+    digit_struct *tmp;
+    if (flags & DEBUG_OUTPUT) {
+      fputs("dropping too small potential digits\n", stderr);
+    }
+    /* count sufficiently large digits */
+    for (d = 0; d < potential_digits; d++) {
+      if (digits[d].x2 - digits[d].x1 >= min_char_dims.w &&
+          digits[d].y2 - digits[d].y1 >= min_char_dims.h) {
+        if (flags & DEBUG_OUTPUT) {
+          fprintf(stderr, " keeping sufficiently large digit %d\n", d);
+        }
+        digit_count += 1;
+      } else if (flags & DEBUG_OUTPUT) {
+        fprintf(stderr, " dropping too small potential digit %d\n", d);
+      }
+    }
+    if (flags & DEBUG_OUTPUT) {
+      fprintf(stderr, "keeping %d of %d potential digits\n", digit_count,
+              potential_digits);
+    }
+    /* at least one digit is required */
+    if (digit_count < 1) {
+      fputs(PROG ": error: no sufficiently large digits found\n", stderr);
+      exit(1);
+    }
+    /* ensure we do not try to keep more digits than we have found */
+    if(digit_count > potential_digits) {
+      fprintf(stderr,
+              PROG ": error: trying to keep more digits (%d) than found (%d)\n",
+              digit_count, potential_digits);
+      exit(99);
+    }
+    /* if potential digits are discarded, copy remaining ones to new memory */
+    if(digit_count < potential_digits) {
+      /* allocate memory for sufficiently large digits we want to keep */
+      if(!(tmp = calloc(digit_count, sizeof(digit_struct)))) {
+        perror(PROG ": tmp = calloc()");
+        exit(99);
+      }
+      /* keep only sufficiently large digits */
+      pos = 0;
+      for (d = 0; d < potential_digits; d++) {
+        if (digits[d].x2 - digits[d].x1 >= min_char_dims.w &&
+            digits[d].y2 - digits[d].y1 >= min_char_dims.h) {
+          if (pos >= digit_count) {
+            fputs(PROG ": error copying digit information", stderr);
+            exit(99);
+          }
+          memcpy(tmp + pos, digits + d, sizeof(digit_struct));
+          pos++;
+        }
+      }
+      free(digits);
+      digits = tmp;
+      potential_digits = digit_count;
+    }
+  }
+
+  /* check if expected number of digits have been found */
+  if ((expected_digits.min > -1) &&
+      ((potential_digits < expected_digits.min) ||
+       (potential_digits > expected_digits.max))) {
+    fprintf(stderr, PROG ": expected between %d and %d digits, but found %d\n",
+            expected_digits.min, expected_digits.max, potential_digits);
+    imlib_free_image_and_decache();
+    if(flags & USE_DEBUG_IMAGE) {
+      save_image("debug", debug_image, output_fmt,debug_image_file,flags);
+      imlib_context_set_image(debug_image);
+      imlib_free_image_and_decache();
+    }
+    exit(1);
+  }
+
+  /* continue to work with the accepted number of characters / digits */
+  number_of_digits = potential_digits;
+  if (flags & DEBUG_OUTPUT) {
+    fprintf(stderr, "image segmentation found %d digits\n", number_of_digits);
+  }
+
+  /* draw rectangles around accepted digits */
   if(flags & USE_DEBUG_IMAGE) {
-  /* draw rectangles around digits */
     imlib_context_set_image(debug_image);
     imlib_context_set_color(128,128,128,255); /* gray line */
     for(d=0; d<number_of_digits; d++) {
@@ -1147,6 +1380,14 @@ int main(int argc, char **argv)
     imlib_context_set_image(image);
   }
 
+  /* determine size of digit part of the image */
+  dig_w = digits[number_of_digits-1].x2 - digits[0].x1;
+  dig_h = digits[number_of_digits-1].y2 - digits[0].y1;
+  if (flags & DEBUG_OUTPUT) {
+    fprintf(stderr, "total width of digit area is %d\n", dig_w);
+    fprintf(stderr, "total height of digit area is %d\n", dig_h);
+  }
+
   /* determine maximum digit dimensions */
   for(d=0; d<number_of_digits; d++) {
     if(max_dig_w < digits[d].x2 - digits[d].x1)
@@ -1160,7 +1401,7 @@ int main(int argc, char **argv)
 
   /* debug: write digit info to stderr */
   if(flags & DEBUG_OUTPUT) {
-    fprintf(stderr, "found %d digits\n", d);
+    fprintf(stderr, "found %d digits\n", number_of_digits);
     for(d=0; d<number_of_digits; d++) {
       fprintf(stderr, "digit %d: (%d,%d) -> (%d,%d), width: %d (%5.2f%%) "
                       "height: %d (%5.2f%%)\n",
@@ -1196,10 +1437,11 @@ int main(int argc, char **argv)
   if(flags & DEBUG_OUTPUT)
     fputs("looking for digit 1\n",stderr);
   for(d=0; d<number_of_digits; d++) {
-    /* skip digits with zero width */
-    if(digits[d].x1 == digits[d].x2) {
+    /* skip digits too narrow for a segment */
+    if(digits[d].x2 - digits[d].x1 < min_segment) {
       if(flags & DEBUG_OUTPUT)
-        fprintf(stderr, " skipping digit %d with zero width\n", d);
+        fprintf(stderr, " skipping too narrow digit %d with width %d\n", d,
+                digits[d].x2 - digits[d].x1);
       continue;
     }
     /* if width of digit is less than 1/one_ratio of its height it is a 1
@@ -1241,10 +1483,11 @@ int main(int argc, char **argv)
   if(flags & DEBUG_OUTPUT)
     fputs("looking for minus signs\n",stderr);
   for(d=0; d<number_of_digits; d++) {
-    /* skip digits with zero height */
-    if(digits[d].y1 == digits[d].y2) {
+    /* skip digits too short for a segment */
+    if(digits[d].y2 - digits[d].y1 < min_segment) {
       if(flags & DEBUG_OUTPUT)
-        fprintf(stderr, " skipping digit %d with zero height\n", d);
+        fprintf(stderr, " skipping too short digit %d with height %d\n", d,
+                digits[d].y2 - digits[d].y1);
       continue;
     }
     /* if height of digit is less than 1/minus_ratio of its height it is a 1
@@ -1263,13 +1506,16 @@ int main(int argc, char **argv)
   }
 
   /* now the digits are located and they have to be identified */
+  if(flags & DEBUG_OUTPUT)
+    fputs("starting scanline based recognition for remaining digits\n", stderr);
   /* iterate over digits */
   for(d=0; d<number_of_digits; d++) {
     int d_height=0; /* height of digit */
-    /* skip digits with zero width or height */
-    if((digits[d].x1 == digits[d].x2) || (digits[d].y1 == digits[d].y2)) {
+    /* skip digits too small to contain a segment */
+    if((digits[d].x2 - digits[d].x1 < min_segment) ||
+       (digits[d].y2 - digits[d].y1 < min_segment)) {
       if(flags & DEBUG_OUTPUT)
-        fprintf(stderr, " skipping digit %d with zero width or height\n", d);
+        fprintf(stderr, " skipping digit %d smaller than minimum segment\n", d);
       continue;
     }
     /* skip already recognized digits */
