@@ -14,7 +14,7 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* Copyright (C) 2004-2022 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
+/* Copyright (C) 2004-2023 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
 
 #ifndef SSOCR2_H
 #define SSOCR2_H
@@ -27,4 +27,12 @@ typedef struct {
   int R, G, B, A;
 } color_struct;
 
+typedef struct {
+  int w, h;
+} dimensions_struct;
+
+typedef struct {
+  int min, max;
+} interval_struct;
+
 #endif /* SSOCR2_H */
